<?php
/**
 * Woocommerce Wholesale Prices Settings
 *
 * @author      Rymera Web
 * @version     1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'WWP_Settings' ) ) {

    class WWP_Settings extends WC_Settings_Page {

        /*
        * @since WWP 1.11
        * We are adding settings by transferring setting options from WWPP to WWP.
        * These options include "Wholesale Price Text", "Disable coupons for wholesale users" and "Hide Original Price".
        * Note that these options we are still using the wwpp_ prefix to maintain values across WWP and WWPP.
        * 
        * @since WWPP 1.24
        * The setting options will be removed in WWPP and its logic codes.
        * WWP will handle the transferred options in this version.
        */

        /**
         * Constructor.
         */
        public function __construct() {

            $this->id    = 'wwp_settings';
            $this->label = __( 'Wholesale Prices', 'woocommerce-wholesale-prices' );

            add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_settings_page' ), 30 ); // 30 so it is after the emails tab
            add_action( 'woocommerce_settings_' . $this->id, array( $this, 'output' ) );
            add_action( 'woocommerce_settings_save_' . $this->id, array( $this, 'save' ) );
            add_action( 'woocommerce_sections_' . $this->id, array( $this, 'output_sections' ) );

            add_action( 'woocommerce_admin_field_upgrade_content' , array( $this, 'render_upgrade_content' ) );

            // Remove upgrade banner tab when wwpp is active
            add_filter( 'wwp_filter_settings_sections' , array( $this, 'remove_upgrade_tab' ) );

            // Set Settings Tab Content
            add_filter( 'wwp_settings_section_content' , array( $this , 'plugin_settings_section_content' ) , 10 , 2 );
            
            do_action( 'wwp_settings_construct' );

        }

        /**
         * Get sections.
         *
         * @return array
         * @since 1.0.0
         */
        public function get_sections() {

            $sections = array(
                ''                              =>  apply_filters( 'wwp_filter_settings_general_section_title' , __( 'General' , 'woocommerce-wholesale-prices' ) ),
                'wwpp_setting_price_section'    => __( 'Price' , 'woocommerce-wholesale-prices' ),
                'wwp_upgrade_section'           => __( 'Upgrade' , 'woocommerce-wholesale-prices' ),
            );

            $sections = apply_filters( 'wwp_filter_settings_sections' , $sections );

            return apply_filters( 'woocommerce_get_sections_' . $this->id, $sections );

        }

        /**
         * Output the settings.
         *
         * @since 1.0.0
         */
        public function output() {

            global $current_section;

            $settings = $this->get_settings( $current_section );
            WC_Admin_Settings::output_fields( $settings );

        }

        /**
         * Save settings.
         *
         * @since 1.0.0
         * @since 1.6.0 Passed the current section on the wwp_before_save_settings and wwp_after_save_settings action filters.
         */
        public function save() {

            global $current_section;

            $settings = $this->get_settings( $current_section );

            do_action( 'wwp_before_save_settings' , $settings , $current_section );

            WC_Admin_Settings::save_fields( $settings );

            do_action( 'wwp_after_save_settings' , $settings , $current_section );

        }

        /**
         * Get settings array.
         *
         * @param string $current_section
         *
         * @return mixed
         * @since 1.0.0
         */
        public function get_settings( $current_section = '' ) {

            $settings = array();

            if ( $current_section == '' ) {

                // General Settings
                $wwpGeneralSettings = apply_filters( 'wwp_general_section_settings' , $this->_get_general_section_settings() ) ;
                $settings = array_merge( $settings , $wwpGeneralSettings );

            } else if ( $current_section === 'wwpp_setting_price_section' ) {

                // Price Section
                $wwp_price_settings = apply_filters( 'wwp_price_section_settings' , $this->_get_price_section_settings() );
                $settings = array_merge( $settings , $wwp_price_settings );

            } else if ( $current_section === 'wwp_upgrade_section' ) {

                // Upgrade Section
                $wwp_upgrade_settings = apply_filters( 'wwp_upgrade_section_settings' , $this->_get_upgrade_section_settings() );
                $settings = array_merge( $settings , $wwp_upgrade_settings );
            
            }

            $settings = apply_filters( 'wwp_settings_section_content' , $settings , $current_section );

            return apply_filters( 'woocommerce_get_settings_' . $this->id, $settings, $current_section );

        }
        
        /**
         * General Setting.
         * This setting comes from WWPP. We maintain the prefix wwpp_ to avoid any with duplicate setting value.
         *
         * @since 1.11
         * @access public
         */
        private function _get_general_section_settings() {

            return array(

                array(
                    'name'  =>  __( 'Wholesale Prices Settings' , 'woocommerce-wholesale-prices' ),
                    'type'  =>  'title',
                    'desc'  =>  '',
                    'id'    =>  'wwp_general_settings_section_title'
                ),
                array(
                    'name'      =>  __( 'Disable Coupons For Wholesale Users', 'woocommerce-wholesale-prices' ),
                    'type'      =>  'checkbox',
                    'desc'      =>  __( 'If checked, this will prevent wholesale users from using coupons' , 'woocommerce-wholesale-prices' ),
                    'id'        =>  'wwpp_settings_disable_coupons_for_wholesale_users',
                    'class'     =>  'wwpp_settings_disable_coupons_for_wholesale_users'
                ),
                array(
                    'type'  =>  'sectionend',
                    'id'    =>  'wwp_genera_settings_sectionend'
                )

            );
            
        }
        
        /**
         * Price settings section options. This setting comes from WWPP. We maintain the prefix wwpp_ to avoid any with duplicate setting value.
         *
         * @since 1.11
         * @access public
         * 
         * @return array
         */
        private function _get_price_section_settings() {

            return array(

                array(
                    'name'  =>  __( 'Price Options', 'woocommerce-wholesale-prices' ),
                    'type'  =>  'title',
                    'desc'  =>  '',
                    'id'    =>  'wwp_settings_price_section_title'
                ),

                array(
                    'name'      =>  __( 'Wholesale Price Text' , 'woocommerce-wholesale-prices' ),
                    'type'      =>  'text',
                    'desc'      =>  '',
                    'desc_tip'  =>  __( 'Default is "Wholesale Price:"', 'woocommerce-wholesale-prices' ),
                    'id'        =>  'wwpp_settings_wholesale_price_title_text',
                    'class'     =>  'wwpp_settings_wholesale_price_title_text'
                ),
                
                array(
                    'name'      =>  __( 'Hide Original Price' , 'woocommerce-wholesale-prices' ),
                    'type'      =>  'checkbox',
                    'desc'      =>  __( 'Hide original price instead of showing a crossed out price if a wholesale price is present.', 'woocommerce-wholesale-prices' ),
                    'desc_tip'  =>  '',
                    'id'        =>  'wwpp_settings_hide_original_price',
                    'class'     =>  'wwpp_settings_hide_original_price'
                ),
                
                array(
                    'type'  =>  'sectionend',
                    'id'    =>  'wwp_settings_price_sectionend'
                )

            );

        }

        /**
         * Price settings section options. This setting comes from WWPP. We maintain the prefix wwpp_ to avoid any with duplicate setting value.
         *
         * @since 1.11
         * @access public
         * 
         * @return array
         */
        private function _get_upgrade_section_settings() {

            // Only show Upgrade tab when WWPP is deactivated
            if( WWP_Helper_Functions::is_plugin_active( 'woocommerce-wholesale-prices-premium/woocommerce-wholesale-prices-premium.bootstrap.php' ) )
                return array();

            return array(

                array(
                    'name'  =>  '',
                    'type'  =>  'title',
                    'desc'  =>  '',
                    'id'    =>  'wwp_settings_upgrade_section_title'
                ),

                array(
                    'name'  =>  '',
                    'type'  =>  'upgrade_content',
                    'desc'  =>  '',
                    'id'    =>  'wwp_settings_upgrade_content',
                ),
                
                array(
                    'type'  =>  'sectionend',
                    'id'    =>  'wwp_settings_upgrade_sectionend'
                )

            );

        }

        /*
        |--------------------------------------------------------------------------------------------------------------
        | Compatibility Check with WWPP. Make sure no duplicate settings is showing up in WWPP
        | NOTE: THIS WILL BE REMOVED IN THE FUTURE VERSIONS OF WWP AFTER WWPP 1.24 is released
        |--------------------------------------------------------------------------------------------------------------
        */

        /**
         * Compatibility check lower than WWPP 1.24
         * Note: this will be removed in the future version of WWP.
         *
         * @since 1.11
         * @return array
         */
        public function prices_premium_compatibility_check() {

            if( WWP_Helper_Functions::is_plugin_active( 'woocommerce-wholesale-prices-premium/woocommerce-wholesale-prices-premium.bootstrap.php' ) ) {
                
                $wwp_plugin_data = WWP_Helper_Functions::get_plugin_data( 'woocommerce-wholesale-prices-premium/woocommerce-wholesale-prices-premium.bootstrap.php' );
                
                return version_compare( $wwp_plugin_data[ 'Version' ] , '1.24' , '<' ) ? true : false;

            }

            return false;
            
        }

        /**
         * Render upgrade content
         *
         * @param $value
         * @since 1.11
         */
        public function render_upgrade_content( $value ) {

            ob_start();
            require_once ( WWP_VIEWS_PATH . 'view-wwp-upgrade-upsell.php' );
            echo ob_get_clean();

        }

        /*
         * Remove duplicate settings if WWPP is active and if for WWPP 1.24 below versions
         * Note: this will be removed in the future version of WWP.
         * 
         * @since 1.11
         * @access public
         *
         * @param array  $settings        Settings options.
         * @param string $current_section Current settings section.
         * @return array
         */
        public function plugin_settings_section_content( $settings , $current_section ) {

            if( $this->prices_premium_compatibility_check() ) {
                
                $settings_temp = $settings;

                // General tab
                if ( $current_section === '' ) {
            
                    foreach( $settings_temp as $index => $setting ) {
                        if( in_array( $setting[ 'id' ] , array( 'wwp_general_settings_section_title' , 'wwp_genera_settings_sectionend' ) ) ||
                            ( isset( $setting[ 'class' ] ) &&  $setting[ 'class' ] === 'wwpp_settings_disable_coupons_for_wholesale_users' ) )
                            unset( $settings[ $index ] );

                    }
                    
                }
                
                // Price tab
                if ( $current_section === 'wwpp_setting_price_section' ) {
            
                    foreach( $settings_temp as $index => $setting ) {
                        if( in_array( $setting[ 'id' ] , array( 'wwp_settings_price_section_title' , 'wwp_settings_price_sectionend' ) ) ||
                            ( isset( $setting[ 'class' ] ) &&  ( $setting[ 'class' ] === 'wwpp_settings_wholesale_price_title_text' ||
                            $setting[ 'class' ] === 'wwpp_settings_hide_original_price' ) ) )
                            unset( $settings[ $index ] );

                    }

                }

            }

            return $settings;

        }

        /**
         * Render upgrade content if WWPP is active.
         *
         * @param $sections
         * @since 1.11
         * @return array
         */
        public function remove_upgrade_tab( $sections ) {
            
            if( WWP_Helper_Functions::is_plugin_active( 'woocommerce-wholesale-prices-premium/woocommerce-wholesale-prices-premium.bootstrap.php' ) && isset( $sections[ 'wwp_upgrade_section' ] ) )
                unset( $sections[ 'wwp_upgrade_section' ] );
                
            return $sections;

        }

    }

}

return new WWP_Settings();
